@@ -131,6 +131,7 @@ static void container_move_to_container_from_direction(
 						container, index);
 			}
 			container->width = container->height = 0;
+			container->width_fraction = container->height_fraction = 0;
 		}
 		return;
 	}
@@ -142,6 +143,7 @@ static void container_move_to_container_from_direction(
 			0 : destination->children->length;
 		container_insert_child(destination, container, index);
 		container->width = container->height = 0;
+		container->width_fraction = container->height_fraction = 0;
 		return;
 	}
 
@@ -163,6 +165,7 @@ static void container_move_to_workspace_from_direction(
 		struct sway_container *container, struct sway_workspace *workspace,
 		enum wlr_direction move_dir) {
 	container->width = container->height = 0;
+	container->width_fraction = container->height_fraction = 0;
 
 	if (is_parallel(workspace->layout, move_dir)) {
 		sway_log(SWAY_DEBUG, "Reparenting container (parallel)");
@@ -206,7 +209,7 @@ static void container_move_to_workspace(struct sway_container *container,
 	} else {
 		container_detach(container);
 		container->width = container->height = 0;
-		container->saved_width = container->saved_height = 0;
+		container->width_fraction = container->height_fraction = 0;
 		workspace_add_tiling(workspace, container);
 		container_update_representation(container);
 	}
@@ -232,9 +235,8 @@ static void container_move_to_container(struct sway_container *container,
 	struct sway_workspace *old_workspace = container->workspace;
 
 	container_detach(container);
-	container_remove_gaps(container);
 	container->width = container->height = 0;
-	container->saved_width = container->saved_height = 0;
+	container->width_fraction = container->height_fraction = 0;
 
 	if (destination->view) {
 		container_add_sibling(destination, container, 1);
@@ -386,6 +388,8 @@ static bool container_move_in_direction(struct sway_container *container,
 	return false;
 }
 
+static struct cmd_results *cmd_move_to_scratchpad(void);
+
 static struct cmd_results *cmd_move_container(bool no_auto_back_and_forth,
 		int argc, char **argv) {
 	struct cmd_results *error = NULL;
@@ -503,6 +507,11 @@ static struct cmd_results *cmd_move_container(bool no_auto_back_and_forth,
 		return cmd_results_new(CMD_INVALID, expected_syntax);
 	}
 
+	if (destination->type == N_CONTAINER &&
+			container_is_scratchpad_hidden(destination->sway_container)) {
+		return cmd_move_to_scratchpad();
+	}
+
 	if (container->is_sticky && container_is_floating_or_child(container) &&
 			old_output && node_has_ancestor(destination, &old_output->node)) {
 		return cmd_results_new(CMD_FAILURE, "Can't move sticky "
@@ -673,10 +682,6 @@ static struct cmd_results *cmd_move_in_direction(
 		return cmd_results_new(CMD_FAILURE,
 				"Cannot move workspaces in a direction");
 	}
-	if (container_is_scratchpad_hidden(container)) {
-		return cmd_results_new(CMD_FAILURE,
-				"Cannot move a hidden scratchpad container");
-	}
 	if (container_is_floating(container)) {
 		if (container->fullscreen_mode) {
 			return cmd_results_new(CMD_FAILURE,
@@ -748,10 +753,6 @@ static struct cmd_results *cmd_move_to_position(int argc, char **argv) {
 		return cmd_results_new(CMD_FAILURE, "Only floating containers "
 				"can be moved to an absolute position");
 	}
-	if (container_is_scratchpad_hidden(container)) {
-		return cmd_results_new(CMD_FAILURE,
-				"Cannot move a hidden scratchpad container");
-	}
 
 	if (!argc) {
 		return cmd_results_new(CMD_INVALID, expected_position_syntax);
@@ -793,6 +794,10 @@ static struct cmd_results *cmd_move_to_position(int argc, char **argv) {
 			ly = root->y + (root->height - container->height) / 2;
 		} else {
 			struct sway_workspace *ws = container->workspace;
+			if (!ws) {
+				struct sway_seat *seat = config->handler_context.seat;
+				ws = seat_get_focused_workspace(seat);
+			}
 			lx = ws->x + (ws->width - container->width) / 2;
 			ly = ws->y + (ws->height - container->height) / 2;
 		}
@@ -826,8 +831,13 @@ static struct cmd_results *cmd_move_to_position(int argc, char **argv) {
 	}
 
 	if (!absolute) {
-		lx += container->workspace->x;
-		ly += container->workspace->y;
+		struct sway_workspace *ws = container->workspace;
+		if (!ws) {
+			struct sway_seat *seat = config->handler_context.seat;
+			ws = seat_get_focused_workspace(seat);
+		}
+		lx += ws->x;
+		ly += ws->y;
 	}
 	container_floating_move_to(container, lx, ly);
 	return cmd_results_new(CMD_SUCCESS, NULL);
@@ -856,7 +866,7 @@ static struct cmd_results *cmd_move_to_scratchpad(void) {
 	}
 
 	if (!con->scratchpad) {
-		root_scratchpad_add_container(con);
+		root_scratchpad_add_container(con, NULL);
 	} else if (con->workspace) {
 		root_scratchpad_hide(con);
 	}
