@@ -17,6 +17,7 @@
 #include <wlr/types/wlr_output.h>
 #include "sway/input/input-manager.h"
 #include "sway/input/seat.h"
+#include "sway/input/switch.h"
 #include "sway/commands.h"
 #include "sway/config.h"
 #include "sway/criteria.h"
@@ -441,6 +442,11 @@ bool load_main_config(const char *file, bool is_active, bool validating) {
 		config->reloading = true;
 		config->active = true;
 
+		// xwayland can only be enabled/disabled at launch
+		sway_log(SWAY_DEBUG, "xwayland will remain %s",
+				old_config->xwayland ? "enabled" : "disabled");
+		config->xwayland = old_config->xwayland;
+
 		if (old_config->swaybg_client != NULL) {
 			wl_client_destroy(old_config->swaybg_client);
 		}
@@ -515,6 +521,12 @@ bool load_main_config(const char *file, bool is_active, bool validating) {
 	}
 
 	if (is_active) {
+		input_manager_verify_fallback_seat();
+		for (int i = 0; i < config->seat_configs->length; i++) {
+			input_manager_apply_seat_config(config->seat_configs->items[i]);
+		}
+		sway_switch_retrigger_bindings_for_all();
+
 		reset_outputs();
 		spawn_swaybg();
 
@@ -522,11 +534,6 @@ bool load_main_config(const char *file, bool is_active, bool validating) {
 		if (config->swaynag_config_errors.client != NULL) {
 			swaynag_show(&config->swaynag_config_errors);
 		}
-
-		input_manager_verify_fallback_seat();
-		for (int i = 0; i < config->seat_configs->length; i++) {
-			input_manager_apply_seat_config(config->seat_configs->items[i]);
-		}
 	}
 
 	if (old_config) {
@@ -644,7 +651,23 @@ void run_deferred_commands(void) {
 		list_free(res_list);
 		free(line);
 	}
-	transaction_commit_dirty();
+}
+
+void run_deferred_bindings(void) {
+	struct sway_seat *seat;
+	wl_list_for_each(seat, &(server.input->seats), link) {
+		if (!seat->deferred_bindings->length) {
+			continue;
+		}
+		sway_log(SWAY_DEBUG, "Running deferred bindings for seat %s",
+				seat->wlr_seat->name);
+		while (seat->deferred_bindings->length) {
+			struct sway_binding *binding = seat->deferred_bindings->items[0];
+			seat_execute_command(seat, binding);
+			list_del(seat->deferred_bindings, 0);
+			free_sway_binding(binding);
+		}
+	}
 }
 
 // get line, with backslash continuation
