@@ -2,7 +2,7 @@
 #include <assert.h>
 #include <stdbool.h>
 #include <stdlib.h>
-#include <wayland-server.h>
+#include <wayland-server-core.h>
 #include <wlr/backend.h>
 #include <wlr/backend/noop.h>
 #include <wlr/backend/session.h>
@@ -11,7 +11,6 @@
 #include <wlr/types/wlr_data_control_v1.h>
 #include <wlr/types/wlr_export_dmabuf_v1.h>
 #include <wlr/types/wlr_gamma_control_v1.h>
-#include <wlr/types/wlr_gamma_control.h>
 #include <wlr/types/wlr_gtk_primary_selection.h>
 #include <wlr/types/wlr_idle.h>
 #include <wlr/types/wlr_layer_shell_v1.h>
@@ -62,12 +61,14 @@ bool server_init(struct sway_server *server) {
 	server->data_device_manager =
 		wlr_data_device_manager_create(server->wl_display);
 
-	wlr_gamma_control_manager_create(server->wl_display);
 	wlr_gamma_control_manager_v1_create(server->wl_display);
 	wlr_gtk_primary_selection_device_manager_create(server->wl_display);
 
 	server->new_output.notify = handle_new_output;
 	wl_signal_add(&server->backend->events.new_output, &server->new_output);
+	server->output_layout_change.notify = handle_output_layout_change;
+	wl_signal_add(&root->output_layout->events.change,
+		&server->output_layout_change);
 
 	wlr_xdg_output_manager_v1_create(server->wl_display, root->output_layout);
 
@@ -120,6 +121,15 @@ bool server_init(struct sway_server *server) {
 	server->presentation =
 		wlr_presentation_create(server->wl_display, server->backend);
 
+	server->output_manager_v1 =
+		wlr_output_manager_v1_create(server->wl_display);
+	server->output_manager_apply.notify = handle_output_manager_apply;
+	wl_signal_add(&server->output_manager_v1->events.apply,
+		&server->output_manager_apply);
+	server->output_manager_test.notify = handle_output_manager_test;
+	wl_signal_add(&server->output_manager_v1->events.test,
+		&server->output_manager_test);
+
 	wlr_export_dmabuf_manager_v1_create(server->wl_display);
 	wlr_screencopy_manager_v1_create(server->wl_display);
 	wlr_data_control_manager_v1_create(server->wl_display);
@@ -161,17 +171,6 @@ void server_fini(struct sway_server *server) {
 }
 
 bool server_start(struct sway_server *server) {
-	// TODO: configurable cursor theme and size
-	int cursor_size = 24;
-	const char *cursor_theme = NULL;
-
-	char cursor_size_fmt[16];
-	snprintf(cursor_size_fmt, sizeof(cursor_size_fmt), "%d", cursor_size);
-	setenv("XCURSOR_SIZE", cursor_size_fmt, 1);
-	if (cursor_theme != NULL) {
-		setenv("XCURSOR_THEME", cursor_theme, 1);
-	}
-
 #if HAVE_XWAYLAND
 	if (config->xwayland) {
 		sway_log(SWAY_DEBUG, "Initializing Xwayland");
@@ -186,17 +185,7 @@ bool server_start(struct sway_server *server) {
 
 		setenv("DISPLAY", server->xwayland.wlr_xwayland->display_name, true);
 
-		server->xwayland.xcursor_manager =
-			wlr_xcursor_manager_create(cursor_theme, cursor_size);
-		wlr_xcursor_manager_load(server->xwayland.xcursor_manager, 1);
-		struct wlr_xcursor *xcursor = wlr_xcursor_manager_get_xcursor(
-			server->xwayland.xcursor_manager, "left_ptr", 1);
-		if (xcursor != NULL) {
-			struct wlr_xcursor_image *image = xcursor->images[0];
-			wlr_xwayland_set_cursor(server->xwayland.wlr_xwayland, image->buffer,
-				image->width * 4, image->width, image->height, image->hotspot_x,
-				image->hotspot_y);
-		}
+		/* xcursor configured by the default seat */
 	}
 #endif
 
