@@ -1,7 +1,7 @@
 #define _POSIX_C_SOURCE 200809L
 #include <stdlib.h>
 #include <strings.h>
-#include <wayland-server.h>
+#include <wayland-server-core.h>
 #include <wlr/render/wlr_renderer.h>
 #include <wlr/types/wlr_buffer.h>
 #include <wlr/types/wlr_output_layout.h>
@@ -183,14 +183,6 @@ bool view_is_only_visible(struct sway_view *view) {
 }
 
 static bool gaps_to_edge(struct sway_view *view) {
-	struct sway_container *con = view->container;
-	while (con) {
-		if (con->current_gaps.top > 0 || con->current_gaps.right > 0 ||
-				con->current_gaps.bottom > 0 || con->current_gaps.left > 0) {
-			return true;
-		}
-		con = con->parent;
-	}
 	struct side_gaps gaps = view->container->workspace->current_gaps;
 	return gaps.top > 0 || gaps.right > 0 || gaps.bottom > 0 || gaps.left > 0;
 }
@@ -232,14 +224,14 @@ void view_autoconfigure(struct sway_view *view) {
 
 		if (config->hide_edge_borders == E_BOTH
 				|| config->hide_edge_borders == E_VERTICAL || hide_smart) {
-			con->border_left = con->x - con->current_gaps.left != ws->x;
-			int right_x = con->x + con->width + con->current_gaps.right;
+			con->border_left = con->x != ws->x;
+			int right_x = con->x + con->width;
 			con->border_right = right_x != ws->x + ws->width;
 		}
 		if (config->hide_edge_borders == E_BOTH
 				|| config->hide_edge_borders == E_HORIZONTAL || hide_smart) {
-			con->border_top = con->y - con->current_gaps.top != ws->y;
-			int bottom_y = con->y + con->height + con->current_gaps.bottom;
+			con->border_top = con->y != ws->y;
+			int bottom_y = con->y + con->height;
 			con->border_bottom = bottom_y != ws->y + ws->height;
 		}
 
@@ -612,6 +604,10 @@ void view_map(struct sway_view *view, struct wlr_surface *wlr_surface,
 		&view->surface_new_subsurface);
 	view->surface_new_subsurface.notify = view_handle_surface_new_subsurface;
 
+	if (decoration) {
+		view_update_csd_from_client(view, decoration);
+	}
+
 	if (view->impl->wants_floating && view->impl->wants_floating(view)) {
 		view->container->border = config->floating_border;
 		view->container->border_thickness = config->floating_border_thickness;
@@ -635,10 +631,6 @@ void view_map(struct sway_view *view, struct wlr_surface *wlr_surface,
 	view_update_title(view, false);
 	container_update_representation(view->container);
 
-	if (decoration) {
-		view_update_csd_from_client(view, decoration);
-	}
-
 	if (fullscreen) {
 		container_set_fullscreen(view->container, true);
 		arrange_workspace(view->container->workspace);
@@ -717,7 +709,7 @@ void view_update_size(struct sway_view *view, int width, int height) {
 		con->surface_y = fmax(con->surface_y, con->content_y);
 	}
 	con->surface_width = width;
-	con->surface_width = height;
+	con->surface_height = height;
 }
 
 static const struct sway_view_child_impl subsurface_impl;
